@@ -2,7 +2,7 @@
  *
  *  Model: system bookmarks (e.g., home folder, desktop)
  *
- *  Copyright (c) 2007 Diego Ongaro <ongardie@gmail.com>
+ *  Copyright (c) 2007-2009 Diego Ongaro <ongardie@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,7 +36,8 @@
 #define EXO_API_SUBJECT_TO_CHANGE
 #include <thunar-vfs/thunar-vfs.h>
 
-#define TRASH THUNAR_VFS_CHECK_VERSION(0,4,0)
+#define TRASH          THUNAR_VFS_CHECK_VERSION(0,4,0)
+#define XDG_USER_DIRS  GLIB_CHECK_VERSION(2,14,0)
 
 #define pbg_priv(pbg) ((PBSysData*) pbg->priv)
 
@@ -45,7 +46,7 @@ typedef struct
 
     /* These are the things that might "change" */
     gboolean       check_changed;   /* starts off false to indicate the following are meaningless */
-    gboolean       desktop_exists;
+    gchar         *desktop_dir;     /* NULL => no desktop or desktop is same as home */
 #if TRASH
     gboolean       trash_is_empty;
     ThunarVfsPath *trash_path;
@@ -77,6 +78,51 @@ pbsys_finalize_trash_bookmark(PlacesBookmark *bookmark)
 }
 #endif
 
+#if TRASH
+static gboolean
+pbsys_trash_is_empty(const ThunarVfsInfo *trash_info)
+{
+    if (trash_info->custom_icon == NULL)
+        return FALSE;
+    if (strcmp("user-trash-full", trash_info->custom_icon) == 0)
+        return FALSE;
+    if (strcmp("gnome-fs-trash-full", trash_info->custom_icon) == 0)
+        return FALSE;
+    return TRUE;
+}
+#endif
+
+
+static gchar*
+pbsys_desktop_dir()
+{
+    const gchar *home_dir = xfce_get_homedir();
+    gchar *desktop_dir = NULL;
+
+#if XDG_USER_DIRS
+    /* get the xdg desktop directory, or possibly NULL */
+    desktop_dir = g_strdup(g_get_user_special_dir(G_USER_DIRECTORY_DESKTOP));
+
+    /* if xdg desktop is the same as home, pretend it's not there */
+    if(exo_str_is_equal(desktop_dir, home_dir)){
+        g_free(desktop_dir);
+        return NULL;
+    }
+#endif
+
+    /* fall back to ~/Desktop */
+    if(desktop_dir == NULL)
+        desktop_dir = g_build_filename(home_dir, "Desktop", NULL);
+
+    /* make sure the directory is there */
+    if(!g_file_test(desktop_dir, G_FILE_TEST_IS_DIR)){
+        g_free(desktop_dir);
+        return NULL;
+    }
+
+    return desktop_dir;
+}
+
 static GList*
 pbsys_get_bookmarks(PlacesBookmarkGroup *bookmark_group)
 {
@@ -87,6 +133,7 @@ pbsys_get_bookmarks(PlacesBookmarkGroup *bookmark_group)
     ThunarVfsInfo *trash_info;
 #endif
     const gchar *home_dir = xfce_get_homedir();
+    gchar *desktop_dir;
 
     pbg_priv(bookmark_group)->check_changed = TRUE;
 
@@ -95,7 +142,7 @@ pbsys_get_bookmarks(PlacesBookmarkGroup *bookmark_group)
     /* Home */
     bookmark                = places_bookmark_create((gchar*) g_get_user_name());
     bookmark->uri           = (gchar*) home_dir;
-    bookmark->icon          = "gnome-fs-home";
+    bookmark->icon          = "user-home";
 
     terminal                 = places_create_open_terminal_action(bookmark);
     bookmark->actions        = g_list_prepend(bookmark->actions, terminal);
@@ -115,13 +162,11 @@ pbsys_get_bookmarks(PlacesBookmarkGroup *bookmark_group)
     /* Try for an icon from ThunarVFS to indicate whether trash is empty or not */
     
     trash_info = thunar_vfs_info_new_for_path(pbg_priv(bookmark_group)->trash_path, NULL);
-    if(trash_info->custom_icon != NULL){
+    pbg_priv(bookmark_group)->trash_is_empty = pbsys_trash_is_empty(trash_info);
+    if(trash_info->custom_icon != NULL)
         bookmark->icon = g_strdup(trash_info->custom_icon);
-        pbg_priv(bookmark_group)->trash_is_empty = (strcmp("gnome-fs-trash-full", bookmark->icon) != 0);
-    }else{
-        bookmark->icon = g_strdup("gnome-fs-trash-full");
-        pbg_priv(bookmark_group)->trash_is_empty = FALSE;
-    }
+    else
+        bookmark->icon = g_strdup("user-trash-full");
     thunar_vfs_info_unref(trash_info);
 
     open                     = places_create_open_action(bookmark);
@@ -132,33 +177,31 @@ pbsys_get_bookmarks(PlacesBookmarkGroup *bookmark_group)
 #endif
 
     /* Desktop */
-    bookmark                = places_bookmark_create(_("Desktop"));
-    bookmark->uri           = g_build_filename(home_dir, "Desktop", NULL);
-    bookmark->icon          = "gnome-fs-desktop";
-    bookmark->finalize      = pbsys_finalize_desktop_bookmark;
+    desktop_dir = pbsys_desktop_dir();
+
+    g_free(pbg_priv(bookmark_group)->desktop_dir);
+    pbg_priv(bookmark_group)->desktop_dir = g_strdup(desktop_dir);
+
+    if(desktop_dir != NULL){
+        bookmark                = places_bookmark_create(_("Desktop"));
+        bookmark->uri           = desktop_dir;
+        bookmark->icon          = "user-desktop";
+        bookmark->finalize      = pbsys_finalize_desktop_bookmark;
+
 
-    if(g_file_test(bookmark->uri, G_FILE_TEST_IS_DIR)){
-    
         terminal                 = places_create_open_terminal_action(bookmark);
         bookmark->actions        = g_list_prepend(bookmark->actions, terminal);
         open                     = places_create_open_action(bookmark);
         bookmark->actions        = g_list_prepend(bookmark->actions, open);
         bookmark->primary_action = open;
 
-        pbg_priv(bookmark_group)->desktop_exists = TRUE;
         bookmarks = g_list_append(bookmarks, bookmark);
-
-    }else{
-
-        pbg_priv(bookmark_group)->desktop_exists = FALSE;
-        places_bookmark_destroy(bookmark);
-
     }
-    
+
     /* File System (/) */
     bookmark                = places_bookmark_create(_("File System"));
     bookmark->uri           = "/";
-    bookmark->icon          = "gnome-dev-harddisk";
+    bookmark->icon          = "gtk-harddisk";
 
     terminal                 = places_create_open_terminal_action(bookmark);
     bookmark->actions        = g_list_prepend(bookmark->actions, terminal);
@@ -174,9 +217,9 @@ pbsys_get_bookmarks(PlacesBookmarkGroup *bookmark_group)
 gboolean
 pbsys_changed(PlacesBookmarkGroup *bookmark_group)
 {
-    gchar *uri;
+    gchar *desktop_dir;
 #if TRASH
-    gboolean trash_is_empty = FALSE;
+    gboolean trash_is_empty;
     ThunarVfsInfo *trash_info;
 #endif
 
@@ -184,18 +227,17 @@ pbsys_changed(PlacesBookmarkGroup *bookmark_group)
         return FALSE;
     
     /* Check if desktop now exists and didn't before */
-    uri = g_build_filename(xfce_get_homedir(), "Desktop", NULL);
-    if(g_file_test(uri, G_FILE_TEST_IS_DIR) != pbg_priv(bookmark_group)->desktop_exists){
-        g_free(uri);
+    desktop_dir = pbsys_desktop_dir();
+    if(!exo_str_is_equal(desktop_dir, pbg_priv(bookmark_group)->desktop_dir)){
+        g_free(desktop_dir);
         return TRUE;
     }else
-        g_free(uri);
+        g_free(desktop_dir);
 
 #if TRASH
     /* see if trash gets a different icon (e.g., was empty, now full) */
     trash_info = thunar_vfs_info_new_for_path(pbg_priv(bookmark_group)->trash_path, NULL);
-    if(trash_info->custom_icon != NULL)
-        trash_is_empty = (strcmp("gnome-fs-trash-full", trash_info->custom_icon) != 0);
+    trash_is_empty = pbsys_trash_is_empty(trash_info);
     thunar_vfs_info_unref(trash_info);
     
     if(trash_is_empty != pbg_priv(bookmark_group)->trash_is_empty)
@@ -212,7 +254,9 @@ pbsys_finalize(PlacesBookmarkGroup *bookmark_group)
     thunar_vfs_path_unref(pbg_priv(bookmark_group)->trash_path);
     thunar_vfs_shutdown();
 #endif
-    
+
+    g_free(pbg_priv(bookmark_group)->desktop_dir);
+
     g_free(pbg_priv(bookmark_group));
 
 }
